<?php

/**
* Core_Module
*
* @author Daniel Milde <daniel@milde.cz>
* @copyright Daniel Milde <daniel@milde.cz>
* @license http://www.opensource.org/licenses/gpl-license.php
* @package Core
*
*/

/**
* Core_Module
*
* @author Daniel Milde <daniel@milde.cz>
* @package Core
*/
abstract class Core_Module extends Core_Object
{
	/**
	* $moduleName
	*
	* @var string $moduleName
	*/
	public $moduleName;

	/**
	* $headerTplFile
	*
	* @var string $headerTplFile
	*/
	public $headerTplFile = 'header.tpl.php';

	/**
	* $footerTplFile
	*
	* @var string $footerTplFile
	*/
	public $footerTplFile = 'footer.tpl.php';

	/**
	* $defaultView
	*
	* @var string $defaultView
	*/
	public $defaultView = 'Default';

	/**
	* $views
	*
	* @var string $views
	*/
	public $views = array('Default');

	/**
	* $cache
	*
	* @var int $cache lifetime
	* 0 = model will be not cached
	*
	*/
	public $cache = 0;

	/**
	* $style
	*
	* @var int $style name of style
	* 0 = model will be not cached
	*
	*/
	public $style = 'default';

	/**
	* $rss
	*
	* Array of rss feeds
	*
	* @var array $rss
	*/
	public $rss = array(); //array( array('title' => '', 'src'=> '') )

	/**
	* $js
	*
	* Array of js files which should be included before main content
	*
	* @var array $js
	*/
	public $js = array();

	/**
	* $css
	*
	* Array of css files
	*
	* @var array $css
	*/
	public $css = array(
		'normal' => array(),
		'ie6'    => array(),
		'ie7'    => array(),
		'print'  => array(),
	);

	/**
	 * $helpers
	 *
	 * Array of view helpers used in template
	 * @var String[] $helpers
	 */
	public $helpers = array();

	/**
	* Constructor
	* @access public
	*/
	public function __construct(){
		parent::__construct();
		$this->moduleName = $this->me->getName();
-		$this->tplFile = strtolower($this->moduleName) . '.tpl.php';
		$this->style = $this->config->style;
		$this->setSiteData();

		if (!$this->request->view) $this->request->view = $this->defaultView;
		else {
			if (!in_array($this->request->view, $this->views))
				throw new UnexpectedValueException('View ' . $this->request->view . ' not allowed for ' . $this->moduleName . ' module', 415);
		}
	}

	/**
	* isValid
	*
	* returns true if model is valid core model
	*
	* @param Core_Model $model
	* @return boolean
	*/
	public static function isValid($module){
		return (is_object($module) && $module instanceof Core_Module);
	}

	/**
	* authenticate
	*
	* @return boolean
	*/
	public function authenticate(){
		return TRUE;
	}

	/**
	* checkRights
	*
	* @return boolean
	*/
	public function authorize(){
		return TRUE;
	}

	public function setSiteData(){
		$site_data['description'] = $this->config->description;
		$site_data['keywords'] = $this->config->keywords;
		$site_data['name'] = $this->config->name;
		$site_data['title'] = $this->config->name;

		foreach($this->js as $js){
			$site_data['js'][] = array('src' => APP_URL.'/js/'.$js);
		}

		foreach($this->rss as $rss){
			$site_data['rss'][] = array('src'   => $this->router->genURL($rss['src'], FALSE),
			                            'title' => $rss['title']);
		}

		if (in_array('Rss', $this->views)) {
			$site_data['rss'][] = array('src'   => '?rss',
										'title' => $this->me->getName() . 's');
		}

		foreach($this->css as $type => $arr){
			foreach($arr as $css) {
				$site_data['css'][$type][] = array('src' => APP_URL.'/styles/'.$this->style.'/css/'.$css);
			}
		}

		$this->setData('site', $site_data);
	}

/* ==================== Module methods ===================== */

	public abstract function __default();

	public function beforeEvent(){}
	public function afterEvent(){}
	public function beforeFooter(){}
}
